@@ -1,4 +1,4 @@
-/*
+﻿/*
     SPDX-FileCopyrightText: 2016 Volker Krause <vkrause@kde.org>
     SPDX-FileCopyrightText: 2018 Christoph Cullmann <cullmann@kde.org>
 
@@ -27,10 +27,16 @@ StateData *StateData::detach(State &state)
     return state.d.data();
 }
 
-void StateData::push(Context *context, QStringList &&captures)
+void StateData::push(const Context *const *firstContext, const Context *const *lastContext, QStringList &&captures)
 {
-    Q_ASSERT(context);
-    m_contextStack.push_back(StackValue{context, std::move(captures)});
+    Q_ASSERT(firstContext < lastContext);
+    if (lastContext - firstContext > 1) [[unlikely]] {
+        --lastContext;
+        for (; firstContext < lastContext; ++firstContext) {
+            m_contextStack.push_back(StackValue{*firstContext, captures});
+        }
+    }
+    m_contextStack.push_back(StackValue{*firstContext, std::move(captures)});
 }
 
 bool StateData::pop(int popCount)
@@ -51,13 +57,13 @@ State::State() = default;
 
 State::State(State &&other) noexcept = default;
 
-State::State(const State &other) = default;
+State::State(const State &other) noexcept = default;
 
 State::~State() = default;
 
 State &State::operator=(State &&other) noexcept = default;
 
-State &State::operator=(const State &other) = default;
+State &State::operator=(const State &other) noexcept = default;
 
 bool State::operator==(const State &other) const
 {
